const EventEmitter = require('events')
const net = require('net') ;

class wts_client extends EventEmitter {
	
	constructor(config) {
		super() 

		this.conf = config ;

	}
	
	init_var(){
		
    	this.connected       = false ;
		this.stopped         = false ;
        this.is_init         = false ;
        this.is_sub          = false ;
        this.is_send         = false ;
        this.wait_recv       = false ;
        this.wait_recv_timer = null ;
        this.last_send       = 0 ;
        this.last_recv       = 0 ;
    	this.data            = {} ;
    	this.buffer          = '' ;
        this.buffer_send     = [] ;
		
	}

	//Start/Stop
    start(){

        this.init_var() ;

		//Check Config
        if(this.conf.host == undefined){
            this.send_status('Config Error') ;
            return ;
        }

        //Start TCP Socket
        this.sock_init() ;

        this.sock_connect() ;

    }

    stop(){

        this.stopped = true ;

        this.sock_disconnect() ;
        
        if(this.timer1s) {
            clearInterval(this.timer1s);
        }

    }

    //Socket
    sock_init(){

        this.sock = new net.Socket() ;

        this.sock.on('data' , (d) => { this.sock_recv(d) });
        this.sock.on('close', (d) => { this.sock_close(d) });
        this.sock.on('error', (d) => { this.sock_error(d) });

    }

    sock_connect(){

        this.send_status('Connecting') ;
        this.sock.connect(8080, this.conf.host, () => {
            this.connected = true ;
            this.send_status('Connected') ;
            this.subscribe(true) ;
            this.request_data() ;
            this.check_1s() ;
        });

    }

    sock_reconnect(){

        this.init_var() ;

        this.sock_connect() ;

    }

    sock_disconnect(){
        this.subscribe(false) ;
        this.sock.end() ;
    }

    sock_close(){

        if(global.debug_msg){ console.log('Connection closed') }

        this.send_status('Disconnected') ;

        this.sock_end() ;

    }

    sock_error(error){

        if(global.debug_msg){
            console.log('Connection error');
            console.log(error) ;
        }

    }

    sock_end(){

        this.connected = false ;

        if(this.timer1s) {
            clearInterval(this.timer1s);
        }

        if(this.stopped){ return }

        setTimeout(() => {
            this.sock_reconnect()
        },1800) ;   

    }

    sock_recv(data){

        this.last_recv = 0 ;

        this.buffer += data.toString() 

        var i = 0 ;
        var a = [] ;
        a = this.buffer.split(/\r\n\r\n/) ;

        if(a.length > 1){

            this.buffer = '' ;
            for(i = 0 ; i < a.length ; i++){

                var j = this.parseJSON(a[i]) ;

                if(j == undefined){ break }

                this.recv_json(j) ;

            }
        }

        //Clean Buffer 
        if(i>0){

            this.buffer = '' ;

            for(i = i ; i < a.length ; i++){ this.buffer += a[i] }

        }

        this.sock_wait_recv() ;

    }

    sock_send(){

        if(this.is_send || this.wait_recv || this.buffer_send.length < 1){ return }

        var msg = this.buffer_send.splice(0, 1) ;

        if(msg.length < 1){ return }

        this.is_send   = true ;
        this.wait_recv = true ;

        if(global.debug_msg){
            console.log('Send JSON :');
            console.log(msg[0]) ;
        }

        this.sock.write(msg[0]+'\r\n\r\n', () => {
            this.last_send = 0 ;
            this.is_send   = false ;
        });

        this.sock_wait_recv_timer() ;
        
    }

    sock_wait_recv(){

        if(this.wait_recv){

            this.wait_recv = false ;

            if(this.buffer_send.length>0){
                this.sock_send()
            }

        }
        
    }

    sock_wait_recv_timer(){

        if(this.wait_recv_timer !== null){

            clearTimeout(this.wait_recv_timer) ;
            this.wait_recv_timer = null ;

        }
        
        this.wait_recv_timer = setTimeout(() => {

            this.sock_wait_recv() ;

        }, 2000);

    }

    //Send DATA
    send_json(j){

        this.buffer_send.push(JSON.stringify(j)) ;

        this.sock_send()

    }

    subscribe(b){

        var j = { subscribe : b } ;

        if(this.conf.password !== '' && this.conf.password !== undefined){
            j.password = this.conf.password
        }

        this.send_json(j) ;
        
    }

    ping(){
        this.send_json({ ping : true })
    }

    request_data(domain){

        var url = '/api/get/json/'  ;
        var id  = 'get-data'

        if(domain!==undefined){
            url += domain ;
            id   = 'get-'+domain
        }

        this.send_json({request:{url:url},id:id})

    }

    request_set_json(j,data){

        var o = { 
            request : {
                url  : '/api/set/json/',
                data : [['json',JSON.stringify(j)],['no-save','1']]
            }
        } ;

        if(data !== undefined && this.is_array(data)){

            for(var i = 0 ; i < data.length ; i++){
                o.request.data.push(data[i]) ;
            }

        }

        this.send_json(o);
        
    }

    //Receive DATA
    recv_json(j){

        if(global.debug_msg){
            console.log('Recv JSON :') ;
            console.log(j) ;
        }
        

        //Request Response
        if(j.request !== undefined && j.request.response !== undefined && j.id !== undefined && this.is_string(j.id)){

            if(j.id == 'get-data'){

                this.is_init = true ;

                for(var o in j.request.response){

                    if(o !== 'code'){
                        this.data[o] = j.request.response[o] ;
                        if(global.debug_msg){ console.log('Add : '+o) }
                    }
    
                }

            }
            else if(j.id.indexOf('get-')==0){
                
                var obj = j.id.split('-')
                obj = obj[1] ;

                for(var o in j.request.response){

                    if(o !== 'code'){
                        this.data[obj][o] = j.request.response[o] ;
                        if(global.debug_msg){ console.log(obj+' Add : '+o) }
                    }

                }

            }
            

        }

        //Subscribe
        else if(j.subscribe){

            this.is_sub = true ;

        }

        //Request Error
        else if(j.request !== undefined && j.request.response !== undefined && j.request.response.code !== undefined && j.request.response.code !== 200){

            if(global.debug_msg){
                console.log('Request Error :') ;
                console.log(j.request.response.desc) ;
            }

            this.emit('request-error',j.request)

        }


        //Data
        else if(j.data !== undefined){

            this.refresh_data_object(this.data,j.data) ;

            this.emit('data',j.data)

        }

        //Events
        else if(j.event !== undefined){

            var e = j.event ;

            //Restart
            if(e.restart){
                
                this.emit('event',e) ;

                this.sock.end() ;
                
            }

            //Update settings
            else if(e.setting_update !== undefined && e.setting_update.domain !== undefined && e.setting_update.domain.length){

                if(j.event.setting_update.domain.length>1){
                    this.request_data()
                }
                else{
                    this.request_data(j.event.setting_update.domain[0])
                }

            }

            //Emit
            else{
                this.emit('event',e)
            }
            

            //No data
            if(this.data == undefined){
                this.data = {} ;
                this.request_data() ;
            }

        }

    }

    //Timer
    check_1s(){

        if(this.stopped || !this.connected){ return }

        if(!this.is_init && this.last_send == 3){
            this.request_data() ;
        }

        if(!this.is_sub && this.last_send == 3){
            this.subscribe(true) ;
        }

        //Last send
        if(this.last_send > 15){
            this.ping() ;
            this.last_send = 0 ;
        }
        else{
            this.last_send++ ;
        }

        //Last recv
        if(this.last_recv > 25){
            this.sock_disconnect() ;
        }
        else{
            this.last_recv++ ;
        }

        //Clear buffer
        if(this.last_recv==2){
            this.buffer = '' ;
        }
        else if(this.last_recv == 20){
            this.subscribe(true)
        }

        //Restart timer
        this.timer1s = setTimeout(() => {
            this.check_1s()
        }, 1000);

    }

    //Divers
    send_status(status){

        this.emit('status', status)

    }

    parseJSON(str){
	
		if(typeof str !== 'string'){ return undefined }
		
		var j ;
		
		try{
			j = JSON.parse(str) ;
		}
		catch(e){
			return undefined ;
		}
		
		return j ;
			
	}

    is_string (v){
        return typeof v === 'string' || v instanceof String
    }

    is_array(a){
        return Object.prototype.toString.call(a) === '[object Array]'
    }

    refresh_data_object(data,new_data){

        for(var o in new_data){

            if(data[o] !== undefined && typeof new_data[o] == 'object' && new_data[o] instanceof Object){

                this.refresh_data_object(data[o],new_data[o]) ;

            }
            else{

                data[o] = new_data[o] ;

                if(global.debug_msg){ console.log('Update '+o+' = ' + new_data[o] + ' ('+ (data[o] !== undefined ? 'Not New' : 'New') +')') }

            }

        }

    }
  
	
  
}

module.exports = wts_client