module.exports = {

    i18n_text(t){
        return global.i18n_text(t) ;
    },

    //Divers
    is_array(a){
        return Object.prototype.toString.call(a) === '[object Array]'
    },

    is_string(v){
        return typeof v === 'string' || v instanceof String
    },

    is_number(v){
        return typeof v === 'number' || v instanceof Number
    },

    is_bool(v){
        return typeof v === 'boolean' || v instanceof Boolean
    },

    is_obj(v){
        return (typeof v === 'object' || v instanceof Object) && v !== null
    },
    
    is_date(d){
        return d instanceof Date && !isNaN(d)
    },

    parseJSON(str){
	
		if(typeof str !== 'string'){ return undefined }
		
		var j ;
		
		try{
			j = JSON.parse(str) ;
		}
		catch(e){
			return undefined ;
		}
		
		return j ;
			
	},

    objectGet(obj, path) { 
        if (typeof obj === "undefined" || obj === null) return;
        path = path.split(/[\.\[\]\"\']{1,2}/);
        for (var i = 0, l = path.length; i < l; i++) {
            if (path[i] === "") continue;
            obj = obj[path[i]];
            if (typeof obj === "undefined" || obj === null) return;
        }
        return obj;
    },

    //Status
    get_status_object(status,current){

        switch(status){

            case 'Connected'    : return {connected: true , fill:"green" ,shape:"dot",text: this.i18n_text('connected')    }
            case 'Connecting'   : return {connected: false, fill:"yellow",shape:"dot",text: this.i18n_text('connecting')   }
            case 'Initializing' : return {connected: false, fill:"yellow",shape:"dot",text: this.i18n_text('initializing') }
            case 'Disconnected' : return {connected: false, fill:"red"   ,shape:"dot",text: this.i18n_text('disconnected') }
            case 'Config Error' : return {connected: false, fill:"red"   ,shape:"dot",text: this.i18n_text('config_error') }

        }

        return current ;

    },

    set_status_last(node){

        var dt = new Date() ;
        const dtOptions = {
            month: 'short',
            day: 'numeric',
            hour: 'numeric',
            minute: 'numeric',
            second: "numeric"
        };
        
        var s = Intl.DateTimeFormat("default", dtOptions).format(dt);
        //var s  = dt.toLocaleDateString() + ' ' + dt.toLocaleTimeString() ;

        node.status({connected:true,fill:"green",shape:"dot",text: this.i18n_text('connected') +' | '+ s }) 
    },

    recv_status(node,status){

        var cfg = node.cfg ;

        if((cfg.obj !== 'connexion' && cfg.obj !== '') || !this.is_string(status.text)){ return }

        status = status.text.toLowerCase() ;

        if(status !== 'connected' && status !== 'disconnected'){ return }

        if(node.last_status == status || (cfg.event !== status && cfg.event !== '')){ return }

        node.last_status = status ;

        this.recv_send_output(node,{ connected : status == 'connected' }) ;

    },

    //Recv
    recv_send_output(node,payload){

        node.send({payload: payload});

        this.set_status_last(node) ;

    },

    //Recv Event
    recv_event(node,evt){

        if(global.debug_msg){
            console.log('Node EVENT :') ;
            console.log(evt) ;
        }
        
        var cfg = node.cfg ;

        //All message
        if(cfg.obj == ''){
            this.recv_send_output(node,evt) ;
            return ;
        }

        //All message object
        if(cfg.event == '' && cfg.id == '' && evt[cfg.obj] !== undefined){
            this.recv_send_output(node,evt[cfg.obj]);
            return ;
        }

        switch(cfg.obj){

            case 'alarm'       : return this.recv_event_alarm(node,evt) ;
            case 'heat'        : return this.recv_event_heat(node,evt) ;
            case 'lighting'    : return this.recv_event_light(node,evt) ;
            case 'opening'     : return this.recv_event_opening(node,evt) ;
            case 'action'      : return this.recv_event_action(node,evt) ;
            case 'home_action' : return this.recv_event_home_action(node,evt) ;
            case 'view_list'   : return this.recv_event_view_list(node,evt) ;

        }

    },

    recv_event_alarm(node,evt){

        var cfg = node.cfg ;

        //No Alarm object
        if(evt.alarm == undefined){ return }

        //No select event
        if(cfg.event !== evt.alarm.state && cfg.event !== ''){ return }

        //No select id
        if(cfg.uid !== evt.alarm.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.alarm) ;

    },

    recv_event_heat(node,evt){

        var cfg = node.cfg ;

        //No heat object
        if(evt.heat == undefined){ return }

        if(cfg.event !== ''){

            if(cfg.event == 'set_point' && evt.heat.set_point == undefined){ return }
            if(cfg.event == 'mode'      && evt.heat.mode      == undefined){ return }
            if(cfg.event == 'mode1'     && evt.heat.mode     !== '1'      ){ return }
            if(cfg.event == 'mode2'     && evt.heat.mode     !== '2'      ){ return }
            if(cfg.event == 'mode3'     && evt.heat.mode     !== '3'      ){ return }
            if(cfg.event == 'mode4'     && evt.heat.mode     !== '4'      ){ return }

        }

        //No select id
        if(cfg.uid !== evt.heat.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.heat) ;

    },

    recv_event_light(node,evt){

        var cfg = node.cfg ;

        //No lighting object
        if(evt.lighting == undefined){ return }

        if(cfg.event !== ''){

            //Event test
            if(cfg.event == 'state-on'  && evt.lighting.state !== 'on')     { return }
            if(cfg.event == 'state-off' && evt.lighting.state !== 'off')    { return }
            if(cfg.event == 'level'     && evt.lighting.level  == undefined){ return }
            if(cfg.event == 'color'     && evt.lighting.color  == undefined){ return }

        }

        //No select id
        if(cfg.uid !== evt.lighting.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.lighting) ;

    },

    recv_event_opening(node,evt){

        var cfg = node.cfg ;

        //No opening object
        if(evt.opening == undefined){ return }

        //No select event
        if(cfg.event !== evt.opening.direction && cfg.event !== ''){ return }

        //No select id
        if(cfg.uid !== evt.opening.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.opening) ;

    },

    recv_event_action(node,evt){

        var cfg = node.cfg ;

        //No action object
        if(evt.action == undefined){ return }

        //No select event
        if(cfg.event !== evt.action.state && cfg.event !== ''){ return }

        //No select id
        if(cfg.uid !== evt.action.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.action) ;


    },

    recv_event_home_action(node,evt){

        var cfg = node.cfg ;

        //No home_action object
        if(evt.home_action == undefined){ return }

        //No select event
        if(cfg.event !== evt.home_action.state && cfg.event !== ''){ return }

        //No select id
        if(cfg.uid !== evt.home_action.id && cfg.uid !== ''){ return }

        this.recv_send_output(node,evt.home_action) ;

    },

    recv_event_view_list(node,evt){

        var cfg = node.cfg ;

        //No view_list object
        if(evt.view_list == undefined){ return }

        //No select View
        if(cfg.uid !== evt.view_list.view && cfg.uid !== ''){ return }

        //No select Item
        if(cfg.item !== evt.view_list.item && cfg.item !== ''){ return }

        //No select event
        if(cfg.event !== evt.view_list.btn.toString() && cfg.event !== ''){ return }

        this.recv_send_output(node,evt.view_list) ;

    },

    //Recv DATA
    recv_data(node,data){

        if(global.debug_msg){
            console.log('Node DATA :') ;
            console.log(data) ;
        }
        
        var cfg = node.cfg ;

        //All message
        if(cfg.obj == ''){
            this.recv_send_output(node,data) ;
            return ;
        }

        //All message object
        if(cfg.event == '' && cfg.id == '' && data[cfg.obj] !== undefined){
            this.recv_send_output(node,data[cfg.obj]) ;
            return ;
        }

        switch(cfg.obj){

            case 'screen'  : return this.recv_data_screen(node,data) ;
            case 'buzzer'  : return this.recv_data_buzzer(node,data) ;
            case 'message' : return this.recv_data_message(node,data) ;
            case 'sd'      : return this.recv_data_sd(node,data) ;

        }

    },

    recv_data_screen(node,data){

        var cfg = node.cfg ;
        var sc  = data.screen ;

        //No Screen Object
        if(sc == undefined){ return }

        if(cfg.event == 'screen-on'        && sc.screen_state !== 1){ return }
        if(cfg.event == 'screen-off'       && sc.screen_state !== 0){ return }
        if(cfg.event == 'screen-saver-off' && sc.screen_saver_state !== 0){ return }
        if(cfg.event == 'screen-saver-on'  && sc.screen_saver_state !== 1){ return }
        if(cfg.event == 'screen-id-0'      && sc.screen_id !== 0){ return }
        if(cfg.event == 'screen-id-1'      && sc.screen_id !== 1){ return }
        if(cfg.event == 'screen-id-2'      && sc.screen_id !== 2){ return }
        if(cfg.event == 'screen-id-3'      && sc.screen_id !== 3){ return }
        if(cfg.event == 'screen-id-4'      && sc.screen_id !== 4){ return }
        if(cfg.event == 'screen-id-5'      && sc.screen_id !== 5){ return }
        if(cfg.event == 'screen-id-6'      && sc.screen_id !== 6){ return }

        this.recv_send_output(node,sc) ;

    },

    recv_data_buzzer(node,data){

        var cfg = node.cfg ;

        //No Buzzer Object
        if(data.buzzer == undefined){ return }

        //No select event
        if(cfg.event !== ''){
            if(cfg.event == '1' && data.buzzer.state !== 1){ return }
            if(cfg.event == '0' && data.buzzer.state !== 0){ return }
        }

        this.recv_send_output(node,data.buzzer) ;

    },

    recv_data_message(node,data){

        var cfg = node.cfg ;

        //No Message Object
        if(data.message == undefined){ return }

        //No select event
        if(cfg.event !== ''){
            if(cfg.event == '1' && data.message.state !== 1){ return }
            if(cfg.event == '0' && data.message.state !== 0){ return }
        }

        this.recv_send_output(node,data.message) ;

    },

    recv_data_sd(node,data){

        var cfg = node.cfg ;

        //No SD Object
        if(data.sd == undefined){ return }

        //No select event
        if(cfg.event !== ''){
            if(cfg.event == '1' && data.sd.inserted !== 1){ return }
            if(cfg.event == '0' && data.sd.inserted !== 0){ return }
        }

        this.recv_send_output(node,data.sd) ;

    },

    //Send
    send_on_input(node,msg){

        var cfg = node.cfg ;
        var o   = {} ;
        var i   = 0 ;

        if(cfg.mode == 'send_recv'){

            if(cfg.domain == '' || cfg.domain == undefined){
                o = msg.payload ;
            }
            else{
                o = {} ;
                o[cfg.domain] = msg.payload ;
            }

        }
        else if(cfg.mode == 'send_defined'){

            if(cfg.rules == undefined || !this.is_array(cfg.rules) || cfg.rules.length < 1){ return }

            for(i = 0 ; i < cfg.rules.length ; i++){

                var domain = cfg.rules[i].domain ; 

                if(domain == 'date_time'){

                    o = this.send_data_date_time(o,cfg.rules[i],msg,node)

                }
                else if(domain == 'view_list'){

                    o = this.send_data_view_list(o,cfg.rules[i],msg,node)

                }
                else{

                    o = this.send_data_object(domain,o,cfg.rules[i],msg,node)

                }

            }

        }

        if(o !== undefined && Object.keys(o).length){

            if(o.view_list !== undefined){
                node.server.wts.request_set_json(o,[['update-view','1']]) ;
            }
            else{
                node.server.wts.request_set_json(o) ;
            }

            this.set_status_last(node) ;

        }
        else{
            node.error(this.i18n_text('no_data_send')) ;
        }
        
    },

    //Send DATA - Domain VAR
    domain_var : [

        { domain : 'theme' , v : [
            { n : 'theme' , t : 'int', min : 0, max : 2 }
        ]},

        { domain : 'screen', v : [
            { n : 'screen_state'       , t : 'int', on_off : 1 },
	        { n : 'screen_saver_state' , t : 'int', on_off : 1 },
	        { n : 'screen_id'          , t : 'int', min : 0, max : 7 },
            { n : 'view_id'            , t : 'str' },
	        { n : 'lumosity'           , t : 'int' },
	        { n : 'touch_beep'         , t : 'int', on_off : 1 }
        ]},

        { domain : 'home' , v : [
            { n : 'text1'        , t : 'str' },
	        { n : 'text2'        , t : 'str' },
	        { n : 'text3'        , t : 'str' },
	        { n : 'text2_small'  , t : 'str' },
	        { n : 'text3_small'  , t : 'str' },
	        { n : 'ribbon_bg'    , t : 'int' , rgb : 1, min : 0, max : 65535 },
	        { n : 'ribbon_text'  , t : 'int' , rgb : 1, min : 0, max : 65535 },
	        { n : 'icon1'        , t : 'int' , min : 0, max : 4 },
	        { n : 'icon2'        , t : 'int' , min : 0, max : 4 },
	        { n : 'icon1_custom' , t : 'int/str' , icon_52 : 1, icon_str : 1 },
	        { n : 'icon2_custom' , t : 'int/str' , icon_52 : 1, icon_str : 1 },
        ]},

        { domain : 'home_action' , v : [
            { n : 'state' , t : 'int', table : 1, on_off : 1 },
        ]},

        { domain : 'action' , v : [
            { n : 'text'  , t : 'str' },
            { n : 'state' , t : 'int', table : 1, on_off : 1 },
        ]},

        { domain : 'lighting' , v : [
            { n : 'text'   , t : 'str' },
            { n : 'subtext', t : 'str', table : 1 },
            { n : 'state'  , t : 'int', table : 1, on_off : 1 },
            { n : 'level'  , t : 'int', table : 1, min : 0, max : 100 },
        ]},

        { domain : 'alarm' , v : [
            { n : 'text'   , t : 'str' },
            { n : 'subtext', t : 'str', table : 1 },
            { n : 'state'  , t : 'int', table : 1, min : 0, max : 3, alarm_state : 1 },
        ]},

        { domain : 'heat' , v : [
            { n : 'text'        , t : 'str' },
            { n : 'subtext'     , t : 'str'  , table : 1 },
            { n : 'state'       , t : 'int'  , table : 1, min : 0, max : 4 },
            { n : 'temp'        , t : 'float', table : 1 },
            { n : 'set_point'   , t : 'float', table : 1 },
            { n : 'mode1_state' , t : 'int'  , table : 1, on_off : 1 },
            { n : 'mode2_state' , t : 'int'  , table : 1, on_off : 1 },
            { n : 'mode3_state' , t : 'int'  , table : 1, on_off : 1 },
            { n : 'mode4_state' , t : 'int'  , table : 1, on_off : 1 },
            { n : 'text_heat'   , t : 'str'  , table : 1 },
        ]},

        { domain : 'opening' , v : [
            { n : 'text'   , t : 'str' },
            { n : 'subtext', t : 'str', table : 1 },
        ]},

        { domain : 'message', v : [
            { n : 'show'  , t : 'bit' },
	        { n : 'icon'  , t : 'int', icon_52 : 1 },
	        { n : 'title' , t : 'str' },
	        { n : 'text'  , t : 'str' },
	        { n : 'text2' , t : 'str' },
	        { n : 'delay' , t : 'int' },
        ]},

        { domain : 'buzzer' , v : [
            { n : 'set'      , t : 'int', min : 0, max : 60000 },
            { n : 'interval' , t : 'int', min : 0, max : 5000 },
            { n : 'delay'    , t : 'int', min : 0, max : 5000 },
        ]},

        { domain : 'date_time' , v : [
            { n : 'set'      , t : 'str' },
            { n : 'set_date' , t : 'str' },
            { n : 'set_time' , t : 'str' },
        ]},

        { domain : 'view_list' , v : [
            { n : 'text'   , t : 'str',     table : 1 },
            { n : 'subtext', t : 'str',     table : 1, item : 1 },
            { n : 'val'    , t : 'str',     table : 1, item : 1 },
	        { n : 'icon'   , t : 'int/str', table : 1, item : 1, icon_52 : 1, icon_str : 1 }
        ]}

    ],

    get_domain_obj(domain,onlyV){

        var o ;

        for(i = 0 ; i < this.domain_var.length ; i++){

            if(this.domain_var[i].domain == domain){
                o = this.domain_var[i] ;
                break ;
            }

        }

        if(onlyV){

            if(o == undefined || o.v == undefined){ return [] }

            return o.v ;

        }
        else{
            return o ;
        }
        

    },

    get_domain_var_obj(domain,varName){
        
        var a = this.get_domain_obj(domain,true) ;

        for(i = 0 ; i < a.length ; i++){

            if(a[i].n == varName){
                return a[i] ;
            }

        }

    },

    //Send DATA - Value
    send_data_transform_val(t,v){

        if(t == undefined){ v }

        if(t.on_off || t.t == 'bit'){

            if(this.is_string(v)){

                var low_v = v.toLowerCase() 

                if(low_v == 'on' || low_v == '1' || low_v == 'true'){
                    v = 1
                }
                else if(low_v == 'off' || low_v == '0' || low_v == 'false'){
                    v = 0 
                }

            }
            else if(this.is_bool(v)){

                v = v ? 1 : 0 ;

            }
            

        }
        else if(t.t == 'int' || t.t == 'float'){

            if(this.is_string(v)){

                if(!isNaN(v)){
                    v = (t.t == 'int') ? parseInt(v) : parseFloat(v) ;
                }

            }
            else if(this.is_bool(v)){

                v = v ? 1 : 0 ;

            }

            if(t.t == 'int'){ v = parseInt(v) }

        }
        else if(t.t == 'str'){

            if(!this.is_string(v)){

                v = v.toString() ;

            }

        }
        else if(t.t == 'int/str'){

            if(!isNaN(v)){

                v = parseInt(v)

            }
            else{

                v = v.toString() ;

            }

        }

        return v ;

    },
    
    send_data_check_val(t,v){

        if(v == undefined){ return this.i18n_text('value_undefined') }

        if(t == undefined){ return '' }

        if((t.on_off || t.t == 'bit')){

            if(!this.is_number(v)){ return this.i18n_text('only_accepts_number') }

            if(v !== 0 && v !== 1){ return this.i18n_text('only_accepts_0_1') }

        }
        else if(t.t == 'int' || t.t == 'float'){

            if(!this.is_number(v)){ return this.i18n_text('only_accepts_number') }

            if(t.min !== undefined && v < t.min){ return this.i18n_text('only_accepts_numb_sup').replace('%val',t.min) }

            if(t.max !== undefined && v > t.max){ return this.i18n_text('only_accepts_numb_low').replace('%val',t.max) }

        }
        else if(t.t == 'str' && !this.is_string(v)){

            return this.i18n_text('only_accepts_str')

        }

        return '' ;

    },

    send_data_get_obj_pos(uid,domain,v,data){

        if(uid == undefined || domain == undefined || v == undefined || data == undefined){ return }

        if(data[domain] == undefined || data[domain][v] == undefined){ return }

        var a = [] ;
        var p ;

        switch(domain){

            case 'alarm' :
                p = 1 ;
                if(data.alarm !== undefined){ a = data.alarm.zone }
                break ;
            case 'lighting' :
                p = 4 ;
                if(data.lighting !== undefined){ a = data.lighting.light }
                break ;
            case 'heat' :
                p = 6 ;
                if(data.heat !== undefined){ a = data.heat.device }
                break ;
            case 'opening' :
                p = 2 ;
                if(data.opening !== undefined){ a = data.opening.device }
                break ;
            case 'action' :
                p = 5 ;
                if(data.action !== undefined){ a = data.action.button }
                break ;
            case 'home_action' : 
                p = 5 ;
                if(data.home_action !== undefined){ a = data.home_action.button }
                break ;

        }

        if(p == undefined){ return }

        for(var o in a){

            if(a[o][p] == uid){
                return o ;
            }

        }

    },

    //Send DATA
    send_data_object(obj,o,rule,msg,node){

        //Get var val
        var val ; 

        switch(rule.val_type){

            case 'msg'    : val = this.objectGet(msg,rule.val) ; break ;
            case 'global' : val = this.objectGet(global,rule.val) ; break ;
            default       : val = rule.val ;

        }

        if(val == undefined){ 
            node.error(this.i18n_text('var_val_undefined').replace('%var',obj +'.'+ rule.v) + (rule.val_type == 'msg' || rule.val_type == 'global' ? (' ('+ this.i18n_text('read_from_var').replace('%var',rule.val_type+'.'+rule.val) + ')') : '')) ;
            return o 
        }

        if(rule.uid !== ''){

            var var_type = this.get_domain_var_obj(obj,rule.v) ;

            val = this.send_data_transform_val(var_type,val) ;

            var check_result = this.send_data_check_val(var_type,val)

            if(check_result !== ''){
                node.error(this.i18n_text('var_val_invalid').replace('%var',obj +'.'+ rule.v).replace('%desc',check_result)) ;
                return o 
            }

        }

        //Create domain object if not exists
        if(o[obj] == undefined){ o[obj] = {} }

        //Assign VAR
        if(rule.uid !== '' && rule.uid !== null && rule.uid !== undefined){

            var pos = this.send_data_get_obj_pos(rule.uid,obj,rule.v,node.server.wts.data) ;

            if(pos !== undefined){

                if(o[obj][rule.v] == undefined){ o[obj][rule.v] = {} }

                o[obj][rule.v][pos] = val ;

            }
            
        }
        else{

            if(rule.val_type == 'json'){

                var j = this.parseJSON(val) ;

                if(j == undefined){
                    node.error(this.i18n_text('var_val_invalid_json').replace('%var',obj +'.'+ rule.v))
                    return o
                }

                o[obj][rule.v] = j ;

            }
            else{

                o[obj][rule.v] = val ;

            }
            
        }

        return o ;

    },

    send_data_date_time(o,rule,msg,node){
        
        var dt ;

        //Get var val
        var val ; 

        switch(rule.val_type){

            case 'msg'    : val = this.objectGet(msg,rule.val) ; break ;
            case 'global' : val = this.objectGet(global,rule.val) ; break ;
            default       : val = rule.val ;

        }

        var dt ;

        if(rule.val_type == 'dt'){
            dt = new Date() ;
        }
        else if(this.is_string(val) || this.is_number(val)){
            dt = new Date(val) ;

            if(!this.is_date(dt)){
                node.error(this.i18n_text('date_val_invalid').replace('%val',val)) ;
                return o ;
            }
        }
        else{
            node.error(this.i18n_text('date_val_invalid_only')) ;
            return o ;
        }

        if(o.date_time == undefined){ o.date_time = {} }

        if(rule.v == 'set' || rule.v == 'set_date'){

            o.date_time.year  = dt.getFullYear() ;
            o.date_time.month = dt.getMonth() + 1 ;
            o.date_time.day   = dt.getDate() ;

        }

        if(rule.v == 'set' || rule.v == 'set_time'){

            o.date_time.hour = dt.getHours() ;
            o.date_time.min  = dt.getMinutes() ;
            o.date_time.sec  = dt.getSeconds() ;
            
        }

        return o ;

    },

    send_data_view_list(o,rule,msg,node){

        var obj = 'view_list' ;

        //Check view selected
        if(rule.uid == '' || rule.uid == undefined){
            node.error(this.i18n_text('no_view_selected')) ;
            return o
        }

        //Get Var Val
        var val ; 

        switch(rule.val_type){

            case 'msg'    : val = this.objectGet(msg,rule.val) ; break ;
            case 'global' : val = this.objectGet(global,rule.val) ; break ;
            default       : val = rule.val ;

        }

        if(val == undefined){ 
            node.error(this.i18n_text('var_val_undefined').replace('%var',obj +'.'+ rule.v) + (rule.val_type == 'msg' || rule.val_type == 'global' ? (' ('+ this.i18n_text('read_from_var').replace('%var',rule.val_type+'.'+rule.val) + ')') : '')) ;
            return o 
        }


        var var_type = this.get_domain_var_obj(obj,rule.v) ;

        val = this.send_data_transform_val(var_type,val) ;

        var check_result = this.send_data_check_val(var_type,val)

        if(check_result !== ''){
            node.error(this.i18n_text('var_val_invalid').replace('%var',obj +'.'+ rule.v).replace('%desc',check_result)) ;
            return o 
        }


        //Create domain object if not exists
        if(o.view_list == undefined){ o.view_list = {} }

        if(o.view_list.views == undefined){ o.view_list.views = [] }


        if(rule.v == 'text'){
            o.view_list.views.push({ id : rule.uid , text : val })
            return o ;
        }


        if(rule.item == '' || rule.item ==  null || rule.item == undefined){

            if(rule.val_type == 'json'){

                var j = this.parseJSON(val) ;

                if(j == undefined){
                    node.error(this.i18n_text('var_val_invalid_json').replace('%id',rule.uid))
                    return o
                }

                o.view_list.views.push({id : rule.uid , items : [j] }) ;
            }
            else{
                o.view_list.views.push({ id :rule.uid , items  : [val] }) ;
            }


        }
        else{

            var it = { id : rule.item }
            it[rule.v] = val ;

            o.view_list.views.push({ id :rule.uid , items : [ it ] }) ;

        }

        return o ;

    },


}