var wts_client = require('./domotys-wts-client.js') ;
var wts_utils  = require('./domotys-wts-utils.js') ;

require('events').EventEmitter.defaultMaxListeners = 0 ;

module.exports = function(RED) {

    global.debug_msg = true ;
    global.i18n_text = function(t){

        return RED._('@domotys/node-red-domotys-wts-04/domotys-wts:wts.'+t) ;

    }

    RED.httpAdmin.get('/data', (req, res) => {
		if(req.query.server == undefined){
			res.status(400).send('Missing argument');
		}
		else{
			var t = RED.nodes.getNode(req.query.server);
			if(t == undefined || t.wts == undefined || t.wts.data == undefined){
				res.status(400).send('Not find');
			}
			else{
				res.status(200).send(JSON.stringify(t.wts.data));
			}
		}
    });

    
    //WTS Server
    function RemoteServerNode(n) {
        
        RED.nodes.createNode(this,n);
        var node  = this;
        node.host = n.host;
        node.name = n.name;
        
        node.options = {
            name : node.name,
            host : node.host,
        };
        
        if(this.credentials){
            node.options.password = this.credentials.password;
        }

        //Status
        node.setStatus = function(status) {
            if(status !== node.status.text){
                node.log("Status changed: "+ node.status.text + "-->" + status);

                node.status = wts_utils.get_status_object(status,node.status)
                
                node.emit("statusChanged", node.status);
            }
        };
        node.setStatus("initializing");

        //Error
        node.recv_request_error = function(req){

            var msg = '' ;
            
            if(req.response.desc !== undefined && req.response.desc.length){
                for(var i = 0 ; i < req.response.desc.length; i++){
                    if(i > 0){ msg += '\n' }
                    msg += ' - ' + req.response.desc[i] ;
                }
            }

            node.error(RED._('wts.request_screen_error') + ' \n' + msg) ;
            
        }

        //Start
        node.wts = new wts_client(node.options) ;
        node.wts.on('status',node.setStatus) ;
        node.wts.on('request-error',node.recv_request_error) ;

        //Stop
        node.on("close", function() {
            node.wts.stop() ;
            node.wts.off('status',node.setStatus) ;
            node.wts.off('request-error',node.recv_request_error) ;
        });

        node.wts.start() ;
        
    }
    
    RED.nodes.registerType("wts-server",RemoteServerNode,{
        credentials: {
            password: {type:"password"}
        }
    });

    
    //WTS Recv
    function wts_recv(config) {
        RED.nodes.createNode(this,config);
        this.cfg = config ;
        
        var node = this;
        node.server = RED.nodes.getNode(config.server);
        
        if(node.server==null || node.server==undefined){
            node.status({fill:"red",shape:"dot",text:RED._('wts.no_screen_set')}) ;
            return ;
        }
        else{
            node.status(node.server.status);
        }

        //Server Status
        node.refreshStatus = function(status) {
            node.status(status);
            wts_utils.recv_status(node,status) ;
        }
        node.server.on('statusChanged', node.refreshStatus);

        //Server event
        node.recv_event = function(e){
            wts_utils.recv_event(node,e) ;
        }
        
        //Server Data change
        node.recv_data = function(e){
            wts_utils.recv_data(node,e) ;
        }

        node.server.wts.on('event',node.recv_event) ;
        node.server.wts.on('data' ,node.recv_data) ;
        
    
        node.on("close", function(){
            node.server.off('statusChanged', node.refreshStatus);
            node.server.wts.off('event',node.recv_event) ;
            node.server.wts.off('data' ,node.recv_data) ;
        });
        
        
    }
    
    RED.nodes.registerType("wts-recv",wts_recv);


    //WTS Send
    function wts_send(config) {
        RED.nodes.createNode(this,config);
        this.cfg = config ;
        
        var node = this;
        node.server = RED.nodes.getNode(config.server);
        
        if(node.server==null || node.server==undefined){
            node.status({fill:"red",shape:"dot",text:RED._('wts.no_screen_set')}) ;
            return ;
        }
        else{
            node.status(node.server.status);
        }
        
        //Server Status
        node.refreshStatus = function(status) {
            node.status(status);
        }
        node.server.on('statusChanged', node.refreshStatus);

        //Node Input
        node.on("input", function(msg) {
            if(!node.server.status.connected){ return }
            wts_utils.send_on_input(node,msg) ;
            node.send(msg)
        });
        
    
        node.on("close", function(){
            node.server.off('statusChanged', node.refreshStatus);
        });
        
        
    }
    
    RED.nodes.registerType("wts-send",wts_send);


}